<?php
/*
 * Our controller file.
 *
 * PHP version 5
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the Affero General Public License as published by Affero Inc.,
 * either version 1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful to the Free
 * Software community, but WITHOUT ANY WARRANTY; without even the implied
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the Affero
 * General Public License for more details.
 * 
 * You should have received a copy of the Affero General Public License with this
 * software in the ./AfferoGPL file; if not, write to Affero Inc., 510 Third Street,
 * Suite 225, San Francisco, CA 94107, USA
 *
 * @category   Controller Class
 * @author     Klaus Weiss <klaus.weiss@nanet.at>
 * @copyright  2007 GNU Herds Free Software Association
 * @license    http://www.affero.org/oagpl.html Affero GPL
 * @version    CVS: $Id:$
 * @since      File available since version 1.5
 */

if(!defined('VISIBLE')) exit();

require_once(ROOT.DS.'core'.DS.'view.class.php');

class Controller
{
  public static $AVAILABLE_LANGS = array('enen' => 'en_EN',
					 'frfr' => 'fr_FR',
					 'itit' => 'it_IT',
					 'ptpt' => 'pt_PT',
					 'ruru' => 'ru_RU',
					 'eses' => 'es_ES'
					 );

  public $view = null;

  public $name = null;

  public $action = null;

  public $pageTitle = SITE_NAME;

					
  public function __construct($controller, $action = null, $params = null)
  {
    self::_initErrorHandler();

    if(!SITE_MOD_REWRITE)
      {
	$array = split('/', $controller);
	$controller = $array[1];
	$action = $array[2];
	
	$i = 0;
	foreach($array as $v)
	  {
	    $i++;
 
	    if($i <= 3 or !$v)
		continue;
    
	    if($i == 4)
	      $params .= $v;
	    else
	      $params .= SITE_URL_SEPARATOR.$v;
	  }

	unset($i);
      }

    $array = split(SITE_URL_SEPARATOR, $params);

    $params = array();
    for($i = 0; $i < 5; $i+=2)
      {
	if(!$array[$i+1])
	  continue;

	$params[$array[$i]] = $array[$i+1];
      }

    unset($i);
    unset($array);

    if(!class_exists($controller.'Controller'))
      {
	show_error_site(ERROR_NO_CONTROLLER);
      }

    if(!method_exists($this, $action))
      {
	show_error_site(ERROR_NO_ACTION);
      }


    $this->view = new View($this->name, $this->helpers);

    $this->beforeRender();

    /* Now we call the action. */
    $this->$action();

    if(array_key_exists('lang', $params) and !empty($params['lang']))
      {
	$k = array_search($params['lang'], self::$AVAILABLE_LANGS);

	if($k !== false)
	  {
	    self::_setLanguage($params['lang']);
	    $this->set('pageLanguage', $params['lang']);
	  }
	else
	  {
	    self::_setLanguage(LOCALE_STANDARD);
	    $this->set('pageLanguage', LOCALE_STANDARD);
	  }
      }
    else
      {
	self::_setLanguage(self::_getPreferredLanguage());
	$this->set('pageLanguage', self::_getPreferredLanguage());
      }

    $this->set('pageCLink', SITE_URL);

    $this->view->pageTitle = $this->pageTitle;

    /* Render is the final step in our View class. */
    $this->view->render($action);

    $this->afterRender();
  }

  public function set($property, $value)
  {
    $this->view->pageVars[$property] = $value;
  }
  
  private static function _initErrorHandler()
  {
    set_error_handler(array('Controller', '_phpErrorHandler'), error_reporting());
    set_exception_handler(array('Controller', '_exceptionHandler'));
  }

  private static function _phpErrorHandler($errno, $errstr, $errfile, $errline)
  {
    if(error_reporting() != false)
      {
	/* 
	 * Possible values of $errno
	 *
	 * E_ERROR
	 * E_WARNING
	 * E_PARSE
	 * E_NOTICE
	 * E_CORE_ERROR
	 * E_CORE_WARNING
	 * E_COMPILE_ERROR
	 * E_COMPILE_WARNING
	 * E_USER_ERROR
	 * E_USER_WARNING
	 * E_USER_NOTICE
	 * E_STRICT
	 */
      }
  }

  private static function _exceptionHandler($e)
  { 
  }

  public function beforeRender()
  {
  }

  public function afterRender()
  {
  }

  private static function _getUserLanguages()
  {
    static $languages = null;
    if($languages === null)
      {
	if(!isset($_SERVER['HTTP_ACCEPT_LANGUAGE']))
	  {
	    $languages[0] = 'en';
	  }	
	else
	  {
	    $languages=array();
	   
	    foreach(explode(',',$_SERVER['HTTP_ACCEPT_LANGUAGE']) as $language)
	      {
		$array=split(';q=', trim($language));
		$languages[trim($array[0])] = isset($array[1]) ? (float)$array[1]:1.0;
	      }
	    
	    arsort($languages);
	    $languages=array_keys($languages);
	    
	    if(empty($languages))
	      {
		$languages[0]='en';
	      }
	  }
      }

    return $languages;
  }

  private static function _getPreferredLanguage()
  {
    static $language = null;

    if($language === null)
      {
	$langs = self::_getUserLanguages();
	$lang = explode('-', $langs[0]);

	if(empty($lang[0]) || !ctype_alpha($lang[0]))
	  {
	    $language = 'en';
	  }
	else
	  {
	    $language = $lang[0];
	  }
      }

    if(!array_key_exists($language, self::$AVAILABLE_LANGS))
      {
	return LOCALE_STANDARD;
      }


    return self::$AVAILABLE_LANGS[$language];
  }

  private static function _setLanguage($language = null)
  {
    if($language != null)
      {
	putenv('LANG='.$language);    
	setlocale(LC_ALL, $language);
	bindtextdomain(LOCALE_STANDARD_DOMAIN, LOCALE_ROOT);
	textdomain(LOCALE_STANDARD_DOMAIN);
      }
  }

  private static function _setCookie($key, $value, $time = 30)
  {
    setcookie($key, $value, time()+(86400*$time), '/', SITE_HOST, false, true);
  }

  public function redirect($url, $status = null)
  {
    if(is_array($url))
      {
	$s_url = $url['controller'].'/'.$url['action'].'/'.$url['params'];	
      }
    else
      {
	$s_url = $url;
      }
    
    header('Location: '.$s_url);
  }
}
?>